
// tinymce 6 base configuration for SuperWebMailer/SuperMailingList
// tinymce.custom_config.js > can overrides it

// load user config file
loadJSfile( 'tinymce/' + "tinymce.custom_config.js", "tinymce.custom_config", "", true);


tinymce_editorConfig = function( language = "de", destelementId, Awidth = "100%", Aheight = "600", AToolbar = "toolbar_MyDefaultMailToolbar", AfullPage = true, allow_script_urls = false, bindModifiedEvent = null, bindInitEvent = null ){

  language = language || "de";

  allow_script_urls = allow_script_urls || false;

  let config = tinymce_editorDefaultConfig;
  if(typeof config.plugins !== "string")
    config.plugins = config.plugins.join(" ");

  if(destelementId && destelementId[0] !== "#")
    destelementId = "#" + destelementId;

  config.toolbar_MyDefaultMailToolbar = config.toolbar_MyDefaultToolbar; // MyDefaultMailToolbar => MyDefaultToolbar

  config.documentBaseURL = tinymce.util.URI.getDocumentBaseUrl(document.location);
  config.baseURI = config.documentBaseURL;
  config.baseURL = config.documentBaseURL;
  config.tinymcebaseURL = config.baseURL + "tinymce/";
  config.tinymceScriptbaseURL = config.tinymcebaseURL + "tinymce/js/tinymce";
  config.baseMyPluginsUrl = config.tinymcebaseURL + 'plugins/';

  let uri = new tinymce.util.URI(document.location);
  config.basePath = uri.directory;
  if(config.basePath === "")
    config.basePath = "/";
  config.baseUserFilesPath = config.basePath + 'userfiles/';
  config.bindModifiedEvent = bindModifiedEvent;
  config.bindInitEvent = bindInitEvent;

  if( typeof tinymce_editorUserConfig !== "undefined" ){
    let forbiddenProperties = ["selector", "target", "setup", "language", "init_instance_callback", "images_upload_url", "fullpage", "loadtargetGroups", "quickUpload", "file_picker_callback", "relative_urls", "remove_script_host", "convert_urls", "urlconverter_callback", "bindModifiedEvent"];

    for (var key in tinymce_editorUserConfig) {
      if( !forbiddenProperties.contains(key)  )
        config[key] = tinymce_editorUserConfig[key];
    }

    if(typeof config.plugins === "string"){
      if(config["plugins_add"] && config["plugins_add"].trim() !== ""){
        config.plugins += " " + config["plugins_add"].trim();
      }

      if(config["plugins_remove"] && config["plugins_remove"].trim() !== ""){
        let a = config["plugins_remove"].split(" ");
        for(var i=0; i<a.length; i++){
          config.plugins = config.plugins.replace(a[i] + " ", "").replace(" " + a[i], "");
        }
      }
    }

    if(config["contextmenu_add"] && config["contextmenu_add"].trim() !== ""){
      config.contextmenu += " " + config["contextmenu_add"].trim();
    }

    if(config["contextmenu_remove"] && config["contextmenu_remove"].trim() !== ""){
      let a = config["contextmenu_remove"].split(" ");
      for(var i=0; i<a.length; i++){
        config.contextmenu = config.contextmenu.replace(a[i] + " ", "").replace(" " + a[i], "");
      }
    }

  }

  if(typeof config.premiumPluginsUrl !== "undefined" && config.premiumPluginsUrl !== "")
    loadJSfile(config.premiumPluginsUrl, "tinymce.premium.plugins", "origin", false);

  // set params
  if(typeof language !== "undefined" && destelementId != null)
    config.language = language;

  if(typeof destelementId !== "undefined" && destelementId != null)
    config.selector = destelementId;

  if(typeof Awidth !== "undefined" && Awidth !== null)
    config.width = Awidth;

  if(typeof Aheight !== "undefined" && Aheight!== null)
    config.height = Aheight;

  if(typeof AfullPage !== "undefined" && AfullPage !== null)
    config.fullpage = AfullPage;

  AToolbar = AToolbar || "toolbar_MyDefaultMailToolbar";

  if(typeof AToolbar !== "undefined" && AToolbar !== null){
    let toolbar = config.toolbar_MyDefaultMailToolbar;

    if(AToolbar.indexOf("toolbar_") !== 0)
      AToolbar = "toolbar_" + AToolbar;

    for (var key in config) {
      if(key == AToolbar){
        toolbar = config[key];
        break;
      }
    }
    config.toolbar = toolbar.toolbar;
    config.menu = toolbar.menu;
   }
  // set params /

  config.images_upload_url = config.quickUpload + '?type=Images&command=QuickUpload&tinymce=6&responseType=json&ckCsrfToken=' + tinymce.getCsrfToken();


  if(typeof config.allow_html_data_urls !== "undefined" && config.allow_html_data_urls == true){

    //data:image base64 support
    config.images_upload_handler = function (blobInfo, progress) {

        if( (blobInfo.filename() == "" || blobInfo.filename().indexOf("blobid") === 0) && blobInfo.blob().type.indexOf("image/") == 0 ){
          // pasted code with data/image
          var s = "data:" + blobInfo.blob().type + ";base64," + blobInfo.base64();
          progress(100);

          return promise = new Promise((resolve, reject) => {
            resolve(s);
          });
        }else{
           // https://www.tiny.cloud/docs/tinymce/6/upload-images/

           return new Promise((resolve, reject) => {
             const xhr = new XMLHttpRequest();
             xhr.withCredentials = tinymce.activeEditor.config.images_upload_credentials || true;
             xhr.open('POST', tinymce.activeEditor.config.images_upload_url);

             xhr.upload.onprogress = (e) => {
               progress(e.loaded / e.total * 100);
             };

             xhr.onload = () => {
               if (xhr.status === 403) {
                 reject({ message: 'HTTP Error: ' + xhr.status, remove: true });
                 return;
               }

               if (xhr.status < 200 || xhr.status >= 300) {
                 reject('HTTP Error: ' + xhr.status);
                 return;
               }

               const json = JSON.parse(xhr.responseText);

               if (!json || typeof json.location != 'string') {
                 reject('Invalid JSON: ' + xhr.responseText);
                 return;
               }

               resolve(json.location);
             };

             xhr.onerror = () => {
               reject('Image upload failed due to a XHR Transport error. Code: ' + xhr.status);
             };

             const formData = new FormData();
             formData.append('file', blobInfo.blob(), blobInfo.filename());

             xhr.send(formData);
           });

        }


    }

  }

  config.allow_script_urls = allow_script_urls;

  if(typeof IsSML !== "undefined" && IsSML){
    if(typeof config.plugins === "string")
      config.plugins = config.plugins.replace("templates ", "").replace(" templates", "");
    config.contextmenu = config.contextmenu.replace("targetgroups ", "").replace(" targetgroups", "");
  }

  return config;
}


let tinymce_editorDefaultConfig = {
    selector: "",
    language: "de",

    height: 600,
    width: "100%",

    fullpage: true,
    fullpage_EMailCleaner: true,

    statusbar: false,

    newdocument_content: '<!DOCTYPE html><html><head><meta name="viewport" content="width=device-width, initial-scale=1"><meta name="format-detection" content="telephone=no"><title></title><meta http-equiv="X-UA-Compatible" content="IE=edge"></head><body style="background-color: #FFFF;">&nbsp;</body></html>',

    schema: 'html4',
    force_hex_color: 'always',

    body_class: 'notranslate',

    forced_root_block : 'p',

    allow_conditional_comments: true,
    convert_fonts_to_spans: false,


    plugins: "initialization codemirror tableclonerow externaldialogs templates fullpagemb placeholders pixabay qrcode anchor autolink charmap code codesample emoticons image link lists advlist nonbreaking, searchreplace table visualblocks visualchars wordcount fullscreen insertdatetime preview quickbars help pagebreak template",
    contextmenu: "mnuQRCode image editimage link linkchecker lists configurepermanentpen spellchecker table tableclonerow targetgroups mnuPageBodyProperties codemirror",

    //https://www.tiny.cloud/docs/tinymce/6/menus-configuration-options/#example-the-tinymce-default-menu-items
    toolbar_MyDefaultToolbar: {
                                toolbar: 'placeholdersbutton functionsDialogButton targetGroupsButton undo redo | blocks fontfamily fontsize forecolor backcolor removeformat | bold italic underline strikethrough | link anchor image qrcodeButton pixabayButton table align | checklist numlist bullist indent outdent | emoticons charmap | fullscreen | codemirror textblocksDialogButton loadtemplatesdialogdialogbutton loadfileDialogButton tableclonerow btnPageBodyProperties',

                                menu: {
                                       file: { title: 'File', items: 'newdocument mnuLoadfileDialog mnuloadtemplatesdialog restoredraft | mnuPageBodyProperties codemirror preview | export print | deleteallconversations' },
                                       tools: {title: 'Tools', items: 'spellchecker spellcheckerlanguage | a11ycheck code wordcount | mnuloadtemplatesdialog mnuPlaceholders mnuTargetgroups mnuLoadfileDialog mnuTextblocksDialog mnuFunctionsDialog mnuPixabay mnuQRCode tableclonerow mnuPageBodyProperties' }
                                      }
                              },

    toolbar_MyDefaultMailToolbar: "",

    toolbar_MyDefaultSimpleToolbar: {

                                toolbar: 'undo redo | blocks fontfamily fontsize forecolor backcolor removeformat | bold italic underline strikethrough | link anchor image qrcodeButton pixabayButton table align | checklist numlist bullist indent outdent | emoticons charmap | fullscreen | codemirror loadfileDialogButton',

                                menu: {
                                       file: { title: 'File', items: 'newdocument mnuLoadfileDialog restoredraft | codemirror preview | export print | deleteallconversations' },
                                       tools: {title: 'Tools', items: 'spellchecker spellcheckerlanguage | a11ycheck code wordcount | mnuLoadfileDialog mnuPixabay mnuQRCode tableclonerow' }
                                      }

                                    },

    toolbar_MyDefaultReadOnlyToolbar: {
                                        toolbar: 'fullscreen',
                                        file: { title: 'File', items: 'preview | export print' },

                                      },

    toolbar_MyDefaultMultiLineToolbar: {

                                           toolbar: 'placeholdersbutton functionsDialogButton targetGroupsButton textblocksDialogButton undo redo | blocks fontfamily fontsize forecolor backcolor removeformat | bold italic underline strikethrough | link anchor image qrcodeButton pixabayButton table align | checklist numlist bullist indent outdent | emoticons charmap | fullscreen | codemirror tableclonerow',

                                           menu: {
                                                  file: { title: 'File', items: 'newdocument codemirror' },
                                                  tools: {title: 'Tools', items: 'spellchecker spellcheckerlanguage | a11ycheck code wordcount | mnuPlaceholders mnuTargetgroups mnuTextblocksDialog mnuFunctionsDialog mnuPixabay mnuQRCode tableclonerow' }
                                                 }

                                       },

    usecodemirrorAsCodeEditor: true,

    codemirror: {
      indentOnInit: true, // Whether or not to indent code on init.
      config: {           // CodeMirror config object
      },
      saveCursorPosition: true    // Insert caret marker
    },


    insertdatetime_formats: ["%H:%M:%S","%d.%m.%Y","%Y-%m-%d","%I:%M:%S %p","%D"],

    font_size_formats: '8pt 9pt 10pt 11pt 12pt 14pt 18pt 24pt 36pt 8px 9px 10px 11px 12px 14px 16px 18px 20px 22px 24px 26px 28px 36px 48px 72px',
    font_size_input_default_unit: "px",

    font_family_formats: 'Andale Mono=andale mono,times; Arial=arial,helvetica,sans-serif; Arial Black=arial black,avant garde; Book Antiqua=book antiqua,palatino; Calibri=calibri; Comic Sans MS=comic sans ms,sans-serif; Courier New=courier new,courier; Georgia=georgia,palatino; Helvetica=helvetica; Impact=impact,chicago; Lucida Console=lucida console,courier new,courier; Microsoft Sans Serif=microsoft sans serif,ms sans serif,serif; Segoe UI=segoe ui; Symbol=symbol; Tahoma=tahoma,arial,helvetica,sans-serif; Terminal=terminal,monaco; Times New Roman=times new roman,times; Trebuchet MS=trebuchet ms,geneva; Verdana=verdana,geneva; Webdings=webdings; Wingdings=wingdings,zapf dingbats',

    style_formats: [
      {
        title: 'Bild links ausrichten',
        selector: 'img',
        styles: {
          float: 'left',
          margin: '0 10px 0 10px'
        }
      },
      {
        title: 'Bild rechts ausrichten',
        selector: 'img',
        styles: {
          float: 'right',
          margin: '0 10px 0 10px'
        }
      }
    ],

    style_formats_merge: true,

    allow_html_data_urls: true,
    images_upload_url: "will be set above on merging user config",
    images_upload_credentials : true,
    automatic_uploads: true,
    images_reuse_filename: true,

    image_advtab: true,
    image_uploadtab: true,
    images_file_types: 'jpeg,jpg,jpe,jfi,jif,jfif,png,gif',

    //relative_urls : false, // ==> NO!!
    remove_script_host : false,
    convert_urls : false, // must be false => don't convert image URLs to local userfiles/2/image.png

    urlconverter_callback: (url, node, on_save, name) => {

        if(node === "img" && name === "src" && url.indexOf("userfiles") == 0) // basePath missing?
          if(typeof tinymce.activeEditor !== undefined && typeof tinymce.activeEditor.config !== undefined && typeof tinymce.activeEditor.config.basePath !== undefined ) url = tinymce.activeEditor.config.basePath + url;

        return url;
      },

    link_default_target: '_blank',
    //link_assume_external_targets: "https", ==> problems with placeholder links
    link_context_toolbar: true,
    link_default_protocol: 'https',

    table_use_colgroups: false,
    table_default_attributes: {
      border: '0',
      width: '100%'
    },
    table_default_styles: { width: "100%" },
    table_column_resizing: 'preservetable',

    autosave_ask_before_unload: false,

    quickbars_insert_toolbar: false,

    toolbar_mode: "sliding",


    file_picker_types: 'image',
    file_picker_callback: function (cb, value, meta) {

      //https://www.tiny.cloud/blog/file-browser-implementation/
      if(meta.filetype !== "image") return; // ==> only images supported by Filemananger

      var input = document.createElement('input');
      var token = getnewToken(16);
      input.setAttribute('type', 'hidden');
      input.setAttribute('id', token);
      document.body.appendChild(input);

      input.addEventListener('change', (e) => {
         let file = e.target.value;

         cb(file, { title: e.target.getAttribute("filename") });

         // update SetImageAsCompleteURL => no onchange event
         var elements = document.querySelectorAll("[name=SetImageAsCompleteURL]");
         if(elements && elements.length > 0){
           elements[0].checked = false;
           elements[0].disabled = !(file.indexOf("userfiles") > -1);
           if(document.getElementsByName('dataURI').length > 0)
              document.getElementsByName('dataURI')[0].disabled = !(file.indexOf("userfiles") > -1);
         }

         tinymce.activeEditor.windowManager.close();
      });

      //https://www.tiny.cloud/docs-4x/advanced/creating-custom-dialogs/

      var editor = tinymce.activeEditor;
      var w = editor.windowManager.openUrl({
        title: 'FileManager',
        url: editor.config.filemanagerPath + '?type=Images&tinymce=' + token + '&langCode=' + editor.config.language,
        width: (window.innerWidth * 80 / 100),
        height: (window.innerHeight * 70 / 100)
      });

      // https://github.com/tinymce/tinymce

      var saveEvent = w.close;
      w.close = function(){
         document.body.removeChild(input);
         input = null;
         if(saveEvent)
           saveEvent();
         };
    },

    //////////////////////////
    quickUpload: "ckeditor/filemanager/connectors/php/upload.php",
    filemanagerPath: "ckeditor/filemanager/index.php",
    pixabayPath: "ckeditor/pixabay/index.php",
    resizeImagePath: "ckeditor/filemanager/connectors/php/resize.php",

    functionsDialogPath: "browsefunctions.php",
    textblocksDialogPath: "browsetextblocks.php",
    loadfileDialogPath: "loadfile.php",
    loadtargetGroups: "ajax_gettargetgroups.php",

    templatesPath: 'fcktemplates.php',
    //////////////////////////

    bindModifiedEvent: null,

    // needed events on setup
    setup: (editor) => {
        editor.on('init', (editor) => {

          // we set config to editor
          if(typeof tinymce.currentEditorConfig !== "undefined"){
             editor.target.config = tinymce.currentEditorConfig;
             delete tinymce.currentEditorConfig;
          }else{
            if(typeof editor.target.config === "undefined")
              editor.target.config = tinymce.defaulteditorConfig;
          }

          if(typeof editor.target.id !== "undefined")
            editor.target.name = editor.target.id;

          tinymce_helper_setup(editor, event);

          if( (typeof _mdialogObj === "undefined" || !_mdialogObj.isVisible()) && document.getElementsByClassName("errortopic").length == 0 )
            try{
              tinymce.activeEditor.focus();
            }catch(e){}

        });
      },

    init_instance_callback: (editor) => {
         tinymce_helper_init_instance_callback(editor);
    },

    get: function(valueName, defaultValue){
      if (typeof this[valueName] !== "undefined")
        return this[valueName]
        else
        return defaultValue;
    },

    set: function(valueName, value){
      this[valueName] = value;
    }

}

// simulate CKEditor Csrf Token for more security
const ckCsrfTokenName = "ckCsrfToken";

function _tinymce_GetCsrfToken(createNew){

   if(typeof createNew === "undefined")
     createNew = false;

   if(!createNew)
     var v = getCookieValue("ckCsrfToken");
   if(!v || createNew)
     v = getnewToken(32);
   return v;
}

tinymce.ckCsrfTokenName = ckCsrfTokenName;
tinymce.getCsrfToken = _tinymce_GetCsrfToken;
setCookieValue(ckCsrfTokenName, tinymce.getCsrfToken(true), 0);
//

// set tinymce.defaulteditorConfig => tinymce_editorConfig()
tinymce.defaulteditorConfig = tinymce_editorConfig();

tinymce.config = function (){ // get config of activeEditor
  if(tinymce.activeEditor)
    return tinymce.activeEditor.config;
  return tinymce.defaulteditorConfig;
};


tinymce.instances = function(instanceName){ //CKEditor "emulation"
  if(typeof instanceName !== "undefined")
    return tinymce.get(instanceName);
    else
    return tinymce.get();
}

// Load my plugins
let baseMyPluginsUrl = tinymce.defaulteditorConfig.baseMyPluginsUrl;

tinymce.PluginManager.load('initialization', baseMyPluginsUrl + 'initialization/plugin.min.js');
tinymce.PluginManager.load('codemirror', baseMyPluginsUrl + 'codemirror/plugin.min.js');
tinymce.PluginManager.load('externaldialogs', baseMyPluginsUrl + 'externaldialogs/plugin.min.js');
tinymce.PluginManager.load('fullpagemb', baseMyPluginsUrl + 'fullpagemb/plugin.min.js');
tinymce.PluginManager.load('pixabay', baseMyPluginsUrl + 'pixabay/plugin.min.js');
tinymce.PluginManager.load('placeholders', baseMyPluginsUrl + 'placeholders/plugin.min.js');
tinymce.PluginManager.load('tableclonerow', baseMyPluginsUrl + 'tableclonerow/plugin.min.js');
tinymce.PluginManager.load('templates', baseMyPluginsUrl + 'templates/plugin.min.js');
tinymce.PluginManager.load('qrcode', baseMyPluginsUrl + 'qrcode/plugin.min.js');
// Load my plugins /


